
# q-expansion of newform 1617.2.a.t, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1617
weight = 2
poly_data = [-1, -4, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-3, 0, 1], 1]]

hecke_ring_character_values = None
aps_data = [[1, 0, 1], [-1, 0, 0], [-1, 1, 1], [0, 0, 0], [-1, 0, 0], [1, 3, -1], [-2, -4, 2], [3, -1, 1], [4, 0, 2], [-1, -3, 1], [2, -2, 4], [1, -1, 3], [-4, -2, 2], [-4, 4, 2], [-1, -1, -3], [0, -2, 0], [-1, 3, -3], [2, 0, 0], [-1, -5, 1], [-4, 4, 0], [7, -3, 1], [4, -4, 0], [-2, 6, 0], [-10, 0, -4], [0, -2, -4], [2, 4, 2], [6, -2, 2], [9, -3, 3], [10, 0, 6], [2, -8, 4], [-6, -2, 0], [-4, 0, 0], [-2, 0, -6], [8, -4, 0], [-1, 5, 1], [-6, -2, -4], [0, -2, -6], [-1, -5, 1], [16, -4, 2], [-4, 2, -10], [0, 4, 4], [-8, 2, -4], [-2, 2, -8], [-12, -2, -4], [-6, -4, -8], [-4, 0, 2], [-10, 2, -2], [14, 6, -2], [6, 2, 2], [-4, -10, 8], [2, 8, -8], [-5, -5, -3], [1, -1, 7], [-3, 1, 7], [-1, -3, 5], [13, 5, 3], [-14, -8, 0], [23, -1, 1], [4, 14, -6], [7, 1, -7], [7, -5, 13], [0, 6, -2], [8, -4, -4], [-8, 0, 0], [-14, 0, -2], [4, 2, 8], [16, -4, 0], [0, -6, 0], [-4, 8, -8], [-7, 11, -1], [17, -1, -5], [-4, -12, 8], [-2, -10, 6], [6, -12, 2], [13, 1, 3], [12, -4, -4], [8, -6, 2], [10, -12, 6], [4, -2, 4], [-14, 12, -4], [-3, 1, -5], [11, 1, 13], [-7, -7, -1], [8, -6, 14], [-13, 3, -11], [8, 12, -8], [-8, 14, -2], [12, 6, 8], [0, -2, -2], [23, -9, -3], [7, -5, 9], [6, -6, -12], [12, -12, -4], [29, 1, -1], [-11, 17, 3], [-24, 4, 2], [6, -4, 12], [-9, 5, -15], [9, 5, 7], [8, 2, 10], [4, 16, -8], [7, -3, 5], [-8, -8, 2], [10, 0, 0], [20, 0, 12], [10, -8, 10], [-3, 17, 7], [20, 6, -2], [20, -4, 0], [-25, 5, 5], [7, 7, -3], [6, 20, -6], [-18, 12, -8], [-14, 10, -16], [36, 4, 0], [-6, 12, -6], [12, 4, 2], [-7, -7, 7], [-16, 10, 10], [17, -3, 3], [20, -10, 4], [20, -2, -4], [18, 8, -4], [6, -18, -4], [-6, 6, 6], [-30, 4, -4], [-33, 5, -15], [-7, -7, 3], [8, 12, -10], [-2, -4, -4], [-24, 0, -6], [-11, 13, 3], [7, -17, -7], [-27, 3, 7], [-12, -2, -10], [13, 11, -5], [7, 17, -11], [-11, 1, -9], [9, -13, 7], [31, -7, 1], [17, 13, -1], [-3, 19, -13], [-5, -5, 5], [29, -7, -5], [16, 6, -2], [11, 3, 9], [22, 4, 0], [10, 12, 4], [12, 8, 0], [12, 16, -6], [8, -22, 22], [-29, 1, 1], [-11, 9, -17], [-10, 2, -20], [-20, 16, -16], [-22, -14, 6], [0, -4, 10], [-19, 3, -5], [2, -4, 12], [-8, -18, 10], [-22, -2, 2], [3, 5, 9], [18, 10, -2], [1, -11, 11], [-28, 14, -4], [40, -8, -4], [31, 7, -7], [6, -20, 4], [2, -16, 0], [-1, 17, 9], [-7, -11, -1], [-26, 12, -16], [8, -8, -8], [-14, -12, 14], [-10, 14, 14], [34, 4, 4], [4, -28, 6], [-6, -4, -16], [5, 13, -9], [-25, -7, 1], [7, 15, -7], [8, 0, -2], [-10, 8, -20], [-16, 18, 6], [4, -20, -4], [-13, 5, 5], [-28, 6, 2], [-6, -30, 12], [-10, -8, 10], [14, 10, -8], [-10, 8, -8], [31, -13, 21], [-21, -9, 13], [-11, -25, -1], [29, 1, -5], [-19, 19, -1], [2, -12, 2], [-2, -24, 4], [-4, 2, -16], [0, -4, 10], [-10, -24, 16], [21, 13, -5], [36, 6, -4], [-5, 1, -15], [-40, -12, 4], [-22, 0, 0], [4, -20, -8], [23, -13, 5], [6, 4, 24], [-12, -4, -10], [-6, 8, 8], [-19, -9, 7], [1, -23, 3], [-16, 4, 0], [-14, -6, 6], [26, 12, 0], [0, 16, -24], [30, -12, -2], [2, 6, -16], [-26, 12, -6], [-14, 8, 2], [1, -15, 7], [-28, 14, -4], [-11, -19, 11], [-20, 8, 4], [-6, -12, 4], [-3, -21, 19], [-6, 18, -2], [-3, -3, -21], [-21, -9, 13], [2, 4, 20], [9, -11, 11], [-34, 6, -10], [-9, 33, 1], [-48, 0, -2], [34, 2, 14], [-34, -4, -2], [19, 1, 1], [-14, -14, 20], [-9, 23, -7], [-24, 8, 10], [12, 8, 24], [14, 2, 0], [6, 16, 4], [-12, 6, 6], [8, -24, -4], [-4, 20, -16], [-13, -1, -15], [-12, 0, -28], [-7, -15, -13], [14, 32, -24], [-48, -10, 4], [22, -2, -2], [-34, 16, -12], [-2, -4, 8], [-52, 0, -8], [-3, -13, 27], [-23, 21, -1], [-3, 7, 7], [-17, -3, -27], [-14, -14, 22], [44, 0, 8], [-22, -16, -8], [35, -3, 17], [-3, -37, 7], [10, 26, 0], [-22, 20, -24], [-9, 29, -11], [0, -20, 0], [10, 20, 4], [8, 14, -2], [-5, -25, -3], [44, -6, 20], [-2, 2, 4], [-1, -11, -3], [-6, -26, 16], [12, 24, -16], [26, -12, -16], [-2, 32, -14], [8, 16, -10], [12, 24, 10], [40, 0, 0], [2, 30, -16], [3, -19, -3], [63, -5, 9], [24, 4, 2], [-49, -7, 5], [-2, -16, 4], [26, -10, 8], [14, 20, 6], [14, -4, -10], [-28, -24, 16], [-21, -3, 1], [-52, -12, -2], [-20, -2, 10], [-4, 22, -8], [-2, -2, -30], [-38, -8, -8], [4, 20, 2], [7, 27, 9], [25, -25, -5], [9, -1, 3], [8, -4, -22], [-16, 0, 18], [19, 7, 13], [-1, -15, 1], [28, -4, 26], [-12, 14, 4], [-44, -8, -2], [16, 6, 12], [18, 2, -10], [36, -2, 18], [63, 9, 5], [-44, 0, 0], [40, 8, 8], [10, 4, 12], [-4, 8, -4], [56, 0, 0], [40, 6, 20], [38, -20, 16], [23, 19, 13], [-25, 33, 1], [-14, 16, -20], [-24, 32, 12], [-11, -1, 27], [28, 14, -14], [-4, -24, 0], [-32, -8, -22], [-49, -9, 13], [31, -11, -7], [16, 26, -14], [-48, -30, 14], [-1, -17, -27], [10, -22, 32], [-24, 8, -22], [23, 11, -19], [23, 29, -19], [0, 14, 0], [-6, 32, -30], [20, -12, 4], [6, -24, -4], [-6, -12, -14], [10, -28, -2], [25, 25, -1], [-34, -12, -6], [-3, -23, -1], [-2, -20, 8], [40, 4, -16], [4, -20, -8], [30, -4, -4], [28, -24, -4], [13, 31, -25], [-40, -6, 14], [-33, 15, -11], [6, 4, -6], [-62, 8, 6], [-5, 19, 13], [32, -4, -20], [19, 21, -19], [8, 4, -10], [-20, -18, -10], [-40, 10, -18], [-36, -4, -24], [-31, 21, -13], [-4, 16, -8], [-21, -43, 29], [10, -36, 30], [14, 6, -14], [22, -36, 4], [-30, 10, -12], [-42, 6, -16], [-9, 23, -7], [53, -5, -13], [-20, 4, 12], [55, -3, 29], [-19, -7, -1], [-25, 23, 17], [7, 13, -19], [14, -8, 24], [10, -12, 28], [13, 39, -21], [-52, 14, -20], [-2, -10, 18], [-9, -31, 21], [4, -24, -8], [27, 35, -7], [59, -5, 33], [-6, -8, -20], [22, 6, -12], [9, -7, -5], [45, 23, -9], [-58, 12, -24], [-4, 28, 18], [44, -12, 50], [-22, -6, 6], [38, -16, 6], [-1, -33, 5], [-15, 15, 7], [8, 20, 32], [3, 17, -7], [1, -13, -9], [30, 0, -10], [18, 22, -16], [42, -4, 28], [-20, -2, 0], [-59, 5, 11], [35, -15, 13], [6, -28, -12], [-46, -10, -22], [32, 12, -8], [14, 8, 36], [14, -16, -12], [56, -20, 0], [-24, -16, -6], [-6, -8, -12], [31, -31, -11], [-3, 21, -25], [22, 2, -4], [46, -16, -12], [-22, -10, -24], [-14, -32, 18], [77, -17, -9], [46, -14, 32], [-30, 26, -24], [6, 16, 16], [-17, -15, -35], [6, -42, 12], [7, 41, -3], [6, 26, 2], [4, -36, 12]]
